#ifndef M_CORE_DEF_H

#define M_CORE_DEF_H

/* 0. Mammouth specific macros cleanup */
#undef format
#undef unused
#undef deprecated
#undef private
#undef public
#undef likely
#undef unlikely

/* 1. check the compiler settings */
#ifndef __GNUC__

    /* other compilers than gcc -- work in progress */
    #warning Compilers other than GNU GCC are currently *not* supported
    #define format
    #define unused
    #define deprecated
    #define private
    #define public
    #define likely(x) x
    #define unlikely(x) x

#else
    /* gcc settings */
    #ifndef _XOPEN_SOURCE
        #define _XOPEN_SOURCE 500
    #endif
    #ifndef _SVID_SOURCE
        #define _SVID_SOURCE 1
    #endif
    #ifndef _FORTIFY_SOURCE
        #define _FORTIFY_SOURCE 2
    #endif
        #ifndef _THREAD_SAFE
    #define _THREAD_SAFE
    #endif
    #ifndef _REENTRANT
        #define _REENTRANT
    #endif

    /* Mammouth specific macros */
    #define format(f, v) __attribute__ ((format(printf, (f), (v))))
    #define unused __attribute__ ((unused))
    #define deprecated __attribute__ ((deprecated))
    #define private __attribute__((visibility("internal")))
    #define public
    #define likely(x)    __builtin_expect(!!(x), 1)
    #define unlikely(x)  __builtin_expect(!!(x), 0)

    /* handle the dllimport related noise in pthread.h with winegcc */
    #define dllimport
#endif

/* 2. basic fixes for WIN32 includes */
#ifdef WIN32
    #define WIN32_LEAN_AND_MEAN
    #include <windows.h>
    #include <errno.h>
    #ifndef off_t
    #define off_t _off_t
    #endif
    
    #ifndef socklen_t
    #define socklen_t size_t
    #endif
#endif

/* 3. standard ISO C99/POSIX includes */
#include <math.h>
#include <wchar.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <wctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <stddef.h>
#include <stdint.h>
#include <string.h>
#include <stdarg.h>
#include <limits.h>
#include <locale.h>
#include <pthread.h>
#include <inttypes.h>
#include <time.h>

/* 5. project specific macros */
#define STRINGIFY(x)  #x
#define STR(x)  STRINGIFY(x)
#define ERR(c, f) #c "()::" #f  "() @ " __FILE__ ":" STR(__LINE__)
#define die(s) do { fprintf(stderr, (s)); abort(); } while(0)
#define MIN(a,b) ((a) < (b) ? (a) : (b))

/* 6. various OS dependant definitions */
#include "ports/m_ports.h"

/* 7. OpenSSL if required */
#ifdef _ENABLE_SSL
    #include <openssl/bio.h>
    #include <openssl/ssl.h>
    #include <openssl/err.h>
#endif

#endif
