#ifndef M_PORT_MMAP_H

#define M_PORT_MMAP_H

/* -------------------------------------------------------------------------- */
#ifdef WIN32 /* WIN32 mmap() compatibility module */
/* -------------------------------------------------------------------------- */

/* standard mmap() definitions - shamelessly ripped off bits/mman.h */

#define MAP_FAILED ((void *) -1)

/* Protections are chosen from these bits, OR'd together.  The
   implementation does not necessarily support PROT_EXEC or PROT_WRITE
   without PROT_READ.  The only guarantees are that no writing will be
   allowed without PROT_WRITE and no access will be allowed for PROT_NONE. */
#define PROT_READ       0x1             /* Page can be read.  */
#define PROT_WRITE      0x2             /* Page can be written.  */
#define PROT_EXEC       0x4             /* Page can be executed.  */
#define PROT_NONE       0x0             /* Page can not be accessed.  */
/* Sharing types (must choose one and only one of these).  */
#define MAP_SHARED      0x01            /* Share changes.  */
#define MAP_PRIVATE     0x02            /* Changes are private.  */
/* Other flags.  */
#define MAP_FIXED       0x10            /* Interpret addr exactly.  */
#define MAP_ANONYMOUS   0x20            /* Don't use a file.  */
#define MAP_ANON        MAP_ANONYMOUS

public void *mmap(void *start, size_t len, int prot, int flags, int fd,
                  off_t offset                                         );
public int munmap(void *start, unused size_t _dummy);

/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
#else /* Standard mmap() on SVID compliant systems */
/* -------------------------------------------------------------------------- */

#define _SVID_SOURCE 1
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h> /* S_IRUSR etc... */
#include <fcntl.h> /* O_* consts in shm_open(2) */

#ifndef MAP_ANON
#define MAP_ANON MAP_ANONYMOUS
#endif

#if ! defined(_SC_PAGESIZE) && defined(_SC_PAGE_SIZE)
#define _SC_PAGESIZE _SC_PAGE_SIZE
#endif

/* -------------------------------------------------------------------------- */
#endif
/* -------------------------------------------------------------------------- */

/* common definitions */
public inline int get_page_size(void);

public void *shm_alloc(const char *name, size_t size);
public void *shm_attach(const char *name, size_t size);
public inline void shm_detach(void *start, size_t size);
public void shm_free(const char *name, void *start, size_t size);

#endif
