#ifndef M_PORT_SOCKET_H

#define M_PORT_SOCKET_H

/* -------------------------------------------------------------------------- */
#ifdef WIN32 /* WIN32 Winsock2 compatibility module */
/* -------------------------------------------------------------------------- */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include <mswsock.h>
#include <process.h>

/* map the WIN32 API functions to their BSD counterparts */
#define ioctl(s, i, l) ioctlsocket((s), (i), (u_long *) (l))

/* map useful Winsock2 error codes to the standard BSD constants */
#undef EINTR
#define EINTR WSAEINTR
#undef EWOULDBLOCK
#define EWOULDBLOCK WSAEWOULDBLOCK
#undef EAGAIN
#define EAGAIN EWOULDBLOCK

/* Winsock2 does not use errno - work around with some macros */
#define ERRNO ( (errno = WSAGetLastError()) )
#define serror(s) (fprintf(stderr, "%s: %s\n", (s), _socket_win32_strerror()))

#if defined(_MSC_VER)
    /* include the needed libs */
    #pragma comment      ( lib, "ws2_32.lib" )
    #pragma comment      ( lib, "pthreadVC2.lib" )
    #if (_MSC_VER < 1300)
        /* Microsoft Visual C++ 6 does not support POSIX networking functions */
        #define _POSIX_EMULATION
    #endif
#elif defined(__GNUC__)
    /* for some reasons, gai_strerror() does not link in Dev-C++ */
    #undef gai_strerror
    #define gai_strerror(i) ("unknown error")
#endif

/* -------------------------------------------------------------------------- */

private const char *_socket_win32_strerror(void);
public size_t socket_sendfile(int out, int in, off_t *off, size_t len);

/* -------------------------------------------------------------------------- */
#else /* *NIX compatibility module */
/* -------------------------------------------------------------------------- */

#include <netdb.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/select.h>

#define serror perror
#define ERRNO errno
#define SOCKET int
#define closesocket(s) close((s))
#define INVALID_SOCKET -1

#if ! defined(TCP_CORK) && defined(TCP_NOPUSH)
    #define TCP_CORK TCP_NOPUSH
#endif

/* sendfile */
#if defined(__linux__)
    #include <sys/sendfile.h>
    #define _socket_sendfile sendfile
#else
    #if defined(__FreeBSD__)
        #include <sys/types.h>
        #include <sys/socket.h>
    #elif defined(__sun)
        #include <sys/sendfile.h>
    #endif

    public size_t socket_sendfile(int out, int in, off_t *off, size_t len);
#endif

/* -------------------------------------------------------------------------- */
#endif
/* -------------------------------------------------------------------------- */

/* -------------------------------------------------------------------------- */
#ifdef _POSIX_EMULATION
/* -------------------------------------------------------------------------- */

/*
  This is a small compatibility layer provided for OS or compilers which do not
  implement the POSIX protocol independant network functions. The assumption is
  made that such an OS/compiler probably does not either properly implement
  IPv6, so only the classic BSD IPv4 API is used for a better portability.
*/

/* define the addrinfo structure */
struct addrinfo {
    int     ai_flags;
    int     ai_family;
    int     ai_socktype;
    int     ai_protocol;
    size_t  ai_addrlen;
    struct sockaddr *ai_addr;
    char   *ai_canonname;
    struct addrinfo *ai_next;
};

/* use the classic sockaddr_in */
#define sockaddr_storage sockaddr_in

/* use the minimalist implementation of the POSIX functions */
#undef getaddrinfo
#undef getnameinfo
#undef freeaddrinfo
#define getaddrinfo  _getaddrinfo
#define getnameinfo  _getnameinfo
#define freeaddrinfo _freeaddrinfo

/* remove gai_strerror */
#undef gai_strerror
#define gai_strerror(i) "unknown error."

/* netdb.h definitions */
#define AI_PASSIVE     0x0001  /* Socket address is intended for `bind'.  */
#define AI_NUMERICHOST 0x0004  /* Don't use name resolution.  */
#define EAI_NONAME       -2    /* NAME or SERVICE is unknown.  */
#define EAI_FAIL         -4    /* Non-recoverable failure in name res.  */
#define EAI_FAMILY       -6    /* `ai_family' not supported.  */
#define EAI_MEMORY       -10   /* Memory allocation failure.  */
#define NI_NUMERICHOST 1       /* Don't try to look up hostname.  */
#define NI_NUMERICSERV 2       /* Don't convert port number to name.  */

/* default size for the host and service buffers */
#define NI_MAXHOST      1025
#define NI_MAXSERV      32

/* -------------------------------------------------------------------------- */

public int getaddrinfo(const char *node, const char *service,
                       const struct addrinfo *hints, struct addrinfo **res);

public int getnameinfo(const struct sockaddr *s, int salen, char *host,
                       size_t hostlen, char *serv, size_t servlen, int flags);

public void freeaddrinfo(struct addrinfo *res);

/* -------------------------------------------------------------------------- */
#endif
/* -------------------------------------------------------------------------- */

#endif
