#include "m_port_std.h"

/* -------------------------------------------------------------------------- */
#ifdef WIN32 /* strtoll()/strtoull() compatibility module */
/* -------------------------------------------------------------------------- */

#ifndef _strtoi64
/* Code from cURL, here is the copyright notice: */

/*
 * COPYRIGHT AND PERMISSION NOTICE
 *
 * Copyright (c) 1996 - 2006, Daniel Stenberg, <daniel@haxx.se>.
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software for any purpose
 * with or without fee is hereby granted, provided that the above copyright
 * notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of a copyright holder shall not
 * be used in advertising or otherwise to promote the sale, use or other dealings
 * in this Software without prior written authorization of the copyright holder.
 *
 */

/**
 * Returns the value of c in the given base, or -1 if c cannot
 * be interpreted properly in that base (i.e., is out of range,
 * is a null, etc.).
 *
 * @param c     the character to interpret according to base
 * @param base  the base in which to interpret c
 *
 * @return  the value of c in base, or -1 if c isn't in range
 */

static int get_char(char c, int base)
{
    int value = -1;
    if (c <= '9' && c >= '0') {
        value = c - '0';
    }
    else if (c <= 'Z' && c >= 'A') {
        value = c - 'A' + 10;
    }
    else if (c <= 'z' && c >= 'a') {
        value = c - 'a' + 10;
    }
    
    if (value >= base) {
        value = -1;
    }
    
    return value;
}

/**
 * Emulated version of the strtoll function.  This extracts a long long
 * value from the given input string and returns it.
 */
long long strtoll(const char *nptr, char **endptr, int base)
{
    char *end;
    int is_negative = 0;
    int overflow;
    int i;
    long long value = 0;
    long long newval;
    
    /* Skip leading whitespace. */
    end = (char *) nptr;
    while (isspace((int) end[0])) {
        end++;
    }
    
    /* Handle the sign, if any. */
    if (end[0] == '-') {
        is_negative = 1;
        end++;
    }
    else if (end[0] == '+') {
        end++;
    }
    else if (end[0] == '\0') {
        /* We had nothing but perhaps some whitespace -- there was no number. */
        if (endptr) {
            *endptr = end;
        }
        return 0;
    }
    
    /* Handle special beginnings, if present and allowed. */
    if (end[0] == '0' && end[1] == 'x') {
        if (base == 16 || base == 0) {
            end += 2;
            base = 16;
        }
    }
    else if (end[0] == '0') {
        if (base == 8 || base == 0) {
            end++;
            base = 8;
        }
    }
    
    /* Matching strtol, if the base is 0 and it doesn't look like
        * the number is octal or hex, we assume it's base 10.
        */
    if (base == 0) {
        base = 10;
    }
    
    /* Loop handling digits. */
    value = 0;
    overflow = 0;
    for (i = get_char(end[0], base);
        i != -1;
        end++, i = get_char(end[0], base)) {
    
        newval = base * value + i;
        if (newval < value) {
            /* We've overflowed. */
            overflow = 1;
            break;
        }
        else
            value = newval;
    }
    
    if (!overflow) {
        if (is_negative) {
            /* Fix the sign. */
            value *= -1;
        }
    }
    else {
        if (is_negative)
            value = LLONG_MIN;
        else
            value = LLONG_MAX;
    
        errno = ERANGE;
    }
    
    if (endptr)
        *endptr = end;
    
    return value;
}
#endif

/* -------------------------------------------------------------------------- */
#endif
/* -------------------------------------------------------------------------- */
