/* gcc -W -Wall -std=c99 -posix -pedantic -O3 rw.c -lpthread -o rw */

#include "../m_core_def.h"

#ifdef __WINE__
#define pthread_self() 0xdeadbeef
#endif

static pthread_rwlock_t lock = PTHREAD_RWLOCK_INITIALIZER;
static char buffer[22] = "hello world !";
static int ctl = 1;

static void *reader(unused void *_dummy)
{
    while (ctl) {
        printf("reader(%x): locking in read mode...\n", (int) pthread_self());
        pthread_rwlock_rdlock(& lock);
        printf("-> reader(%x): locked\n", (int) pthread_self());

        printf("reader(%x): the buffer says: \"%s\"\n", (int) pthread_self(), buffer);
    
        printf("reader(%x): unlocking...\n", (int) pthread_self());
        pthread_rwlock_unlock(& lock);
        printf("-> reader(%x): unlocked\n", (int) pthread_self());
    }

    return NULL;
}

static void *writer(void *text)
{
    const char *txt = (char *) text;

    printf("writer(%x): must write \"%s\"\n", (int) pthread_self(), txt);

    while (ctl) {
        printf("writer(%x): locking in write mode...\n", (int) pthread_self());
        pthread_rwlock_wrlock(& lock);
        printf("----------- writer(%x): locked -----------\n", (int) pthread_self());

        printf("writer(%x): writing in the buffer.\n", (int) pthread_self());
        memcpy(buffer, txt, strlen(txt) + 1);
        printf(
               "writer(%x): wrote \"%s\" in the buffer.\n",
               (int) pthread_self(),
               buffer
              );

        printf("writer(%x): unlocking...\n", (int) pthread_self());
        pthread_rwlock_unlock(& lock);
        printf("---------- writer(%x): unlocked ----------\n", (int) pthread_self());
        
    }

    return NULL;
}
    
int main(void)
{
    pthread_t r1, r2, w1, w2, r3, r4;

    setbuf(stdout, NULL);

    pthread_create(& r1, NULL, reader, NULL);
    pthread_create(& w1, NULL, writer, "goodbye cruel world !");
    pthread_create(& r2, NULL, reader, NULL);
    pthread_create(& w2, NULL, writer, "hello world !");
    pthread_create(& r3, NULL, reader, NULL);
    pthread_create(& r4, NULL, reader, NULL);

    getchar();

    ctl = 0;

    pthread_join(r1, NULL);
    pthread_join(w1, NULL);
    pthread_join(r2, NULL);
    pthread_join(w2, NULL);
    pthread_join(r3, NULL);
    pthread_join(r4, NULL);

    exit(EXIT_SUCCESS);
}
