#include "../m_socket.h"
#include <signal.h>

/* gcc -g -W -Wall -std=c99 -posix -pedantic -O2 ../ports/m_ports.c ../m_string.c ../m_queue.c ../m_socket.c sockets.c -lpthread -lrt -o sock */

/* gcc -g -W -Wall -std=c99 -posix -pedantic -O2 ../ports/m_ports.c ../m_string.c ../m_queue.c ../m_socket.c sockets.c -D_ENABLE_SSL -lssl -lpthread -lrt -o sock */

int main(void)
{
    char buf[BUFSIZ] = { 0 };
    int ret = 0;
    int cx = 0;
    const char *ip = "212.27.33.225";
    const char *req = "GET / HTTP/1.0\r\nUser-Agent: Testing\r\n\r\n";
    #ifdef _ENABLE_SSL
    const char *port = "443";
    m_socket *s = socket_open(ip, port, SOCKET_SSL);
    #else
    const char *port = "80";
    m_socket *s = socket_open(ip, port, 0x0);
    #endif

    signal(SIGPIPE, SIG_IGN);

    if (s == NULL) {
        fprintf(stderr, "Socket creation failed\n"); exit(EXIT_FAILURE);
    }

    printf("Connecting to %s:%s...\n", ip, port);

    ret = socket_connect(s);
    if (ret != 0 && ret != SOCKET_EAGAIN) {
        fprintf(stderr, "Connection failed\n");
        socket_close(s); exit(EXIT_FAILURE);
    }

    do {
        ret = socket_write_lowlevel(s, req, strlen(req));
        cx ++;
    } while (ret == SOCKET_EAGAIN);
    if (ret < 0) {
        if (ret == SOCKET_ECLOSE) {
            printf("Link closed.\n");
            socket_close(s);
            exit(EXIT_FAILURE);
        } else if (ret == SOCKET_EFATAL) {
            printf("Fatal error.\n");
            perror(ERR(main, _socket_write));
            socket_close(s);
            exit(EXIT_FAILURE);
        }
    }

    printf("Wrote %i bytes, called socket_write() %i time(s)\n", ret, cx);

    cx = 0;
    do {
        ret = socket_read_lowlevel(s, buf, sizeof(buf));
        cx ++;
    } while (ret == SOCKET_EAGAIN);
    if (ret < 0) {
        if (ret == SOCKET_ECLOSE) {
            printf("Link closed.\n");
            socket_close(s);
            exit(EXIT_FAILURE);
        } else if (ret == SOCKET_EFATAL) {
            printf("Fatal error.\n");
            perror(ERR(main, _socket_read));
            socket_close(s);
            exit(EXIT_FAILURE);
        }
    }

    printf("Read %i bytes, called socket_read() %i time(s)\n", ret, cx);

    printf("Data received: %s\n", buf);

    s = socket_close(s);

    return 0;
}
