#include "../m_string.h"

/* gcc -g -W -Wall -std=c99 -posix -pedantic -O2 ../m_string.c string.c -o string */

/* winegcc -mno-cygwin -I../ports/libpthread-win32/include -g -Wall -std=c99 -posix -pedantic -O2 ../m_string.c string.c -o string */

int main(void)
{
    m_string *string = NULL, *data = NULL, *tmp = NULL;
    int i = 0, j = 0;

    setlocale(LC_CTYPE, "de_DE");

    printf("* testing string_from_uint32\n");
    i = htonl(0xCAFE);
    tmp = string_from_uint32(i);
    if (tmp)
    printf("-> string successfully allocated\n");
    else
    printf("! *** string_from_uint32 failed ! ***\n");

    printf("* testing string_swap and string_to_uint32\n");
    j = string_swap(tmp);
    if (j == -1)
    printf("! *** string_swap failed ! ***\n");
    else {
    string_to_uint32(tmp, (uint32_t *) & i);
    printf("-> the registered number was 0x%X\n", i);
    }

    i = j = 0; tmp = string_free(tmp);

    printf("* testing string_alloc\n");
    string = string_alloc("comma separated", strlen("comma separated"));
    data = string_alloc(",word list,", strlen(",word list,"));
    printf("-> \"%s\" and \"%s\" were allocated\n", CSTR(string), CSTR(data));

    printf("* testing string_cpy\n");
    tmp = string_cpy(tmp, string);
    printf("-> \"%s\" is a copy of \"%s\"\n", CSTR(tmp), CSTR(string));

    printf("* searching for an inexisting piece of data should return -1... ");
    if (string_finds(tmp, 0, "z", 1) != -1)
    printf("FAILURE\n");
    else
    printf("OK\n");

    printf("* testing aliasing in string_movs\n");
    tmp = string_movs(tmp, 1, CSTR(tmp) + 4, 1);
    printf("-> moving the 5th char of tmp in the 2nd place: %s\n", CSTR(tmp));
    tmp = string_free(tmp);

    printf("* testing string_pre\n");
    data = string_pre(data, string);
    printf("-> \"%s\" was prepended \"%s\"\n", CSTR(data), CSTR(string));

    printf("* testing string_cat\n");
    data = string_cat(data, string);
    printf("-> \"%s\" was appended \"%s\"\n", CSTR(data), CSTR(string));

    printf("* testing string_splits with separator: \"%s\"\n", ",");
    i = string_splits(data, ",", strlen(","));
    if (i != -1) {
        printf("-> found following parts:\n-> ");
        putchar('|');
        for (i = 0; i < PARTS(data); i ++) {
            for (j = 0; j < SIZE(TOKEN(data, i)); j ++)
                putchar(*(CSTR(TOKEN(data, i)) + j));
            putchar('|');
        }
        printf("\n");
        printf("   |_ trying to viciously resize a token... ");
        /* without cast, GCC would remind us that the token is const ... */
        if (string_dim((m_string *) TOKEN(data, 0), 2) == -1)
        printf("failure :)\n");
        else
        printf("success :(\n");
        printf("   |_ trying to mischievously overwrite a token... ");
        if (string_pres((m_string *) TOKEN(data, 0), "bug", strlen("bug")) == NULL)
        printf("failure :)\n");
        else
        printf("success :(\n");
        printf("   |_ splitting first token with: \"%s\"\n", " ");
        i = string_splits((m_string *) TOKEN(data, 0), " ", strlen(" "));
        if (i != -1) {
            printf("      |_ found following parts:\n         ->");
            putchar('|');
            for (i = 0; i < PARTS(TOKEN(data, 0)); i ++) {
                for (j = 0; j < SIZE(TOKEN(TOKEN(data, 0), i)); j ++)
                    putchar(*(CSTR(TOKEN(TOKEN(data, 0), i)) + j));
                putchar('|');
            }
            printf("\n");
        }
        else
        printf("      |_ ! *** string_splits on token failed ! ***\n");
    }
    else
    printf("! *** string_splits failed ! ***\n");

    printf("* testing string_reps: replacing \"%s\" with \"%s\"\n", CSTR(string), "graou !");
    tmp = string_reps(data, CSTR(string), SIZE(string), "graou !", strlen("graou !"));
    if (tmp == NULL) {
    printf("! *** string_reps failed ! ***\n");
    } else {
        data = tmp;
        printf("-> result: \"%s\"\n", CSTR(data));
    }

    printf("* testing substring deletion (\"%s\") with string_rems\n", "graou !");
    tmp = string_rems(data, "graou !", strlen("graou !"));
    if (tmp == NULL) {
    printf("! *** string_reps failed ! ***\n");
    } else {
        data = tmp;
        printf("-> result: \"%s\"\n", CSTR(data));
    }
    
    
    data = string_free(data); string = string_free(string);

    printf("* testing string creation with string_movs\n");
    string = string_pres(string, "Schne Gre", strlen("Schne Gre"));
    printf("-> \"%s\" was created using string_movs\n", CSTR(string));

    printf("* testing string_wchar\n");
    i = string_wchar(string);
    if (i != -1)
    printf("-> WCHAR conversion: \"%ls\"\n", WSTR(string));
    else
    printf("! *** string_wchar failed ! ***\n");

    /* FIXME broken with gcc 3.4.5 */
    /*printf("* testing string_finds on WCHAR\n");
    i = string_finds(string, 0, (char *) L"Gre", wcslen(L"Gre"));
    if (i != -1)
    printf("-> string_finds result: \"%ls\"\n", WSTR(string) + (i / sizeof(wchar_t)));
    else
    printf("! *** string_finds failed ! ***\n");*/

    printf("* testing string_mbyte\n");
    i = string_mbyte(string);
    if (i != -1)
    printf("-> MBYTE conversion: \"%s\"\n", CSTR(string));
    else
    printf("! *** string_mbyte failed ! ***\n");

    printf("* testing string_finds on MBYTE\n");
    i = string_finds(string, 0, "Gre", strlen("Gre"));
    if (i != -1)
    printf("-> string_finds result: \"%s\"\n", CSTR(string) + i);
    else
    printf("! *** string_finds failed ! ***\n");

    printf("* testing string_upper\n");
    i = string_upper(string);
    if (i != -1)
    printf("-> \"%s\" was converted to upper case\n", CSTR(string));
    else
    printf("! *** string_upper failed ! ***\n");

    printf("* testing string_lower\n");
    string_lower(string);
    if (i != -1)
    printf("-> \"%s\" was converted to lower case\n", CSTR(string));
    else
    printf("! *** string_lower failed ! ***\n");

    printf("* testing string_fmt\n");
    tmp = NULL;
    tmp = string_fmt(tmp, "%i - %s", 10, "The thin line between love and hate");
    if (tmp == NULL)
    printf("! *** string_fmt failed ! ***\n");
    else
    printf("-> string_fmt result: \"%s\"\n", CSTR(tmp));

    printf("* testing string_b64 and string_deb64\n");
    string = string_free(string);
    string = string_b64(tmp, 72);
    if (string == NULL)
    printf("! *** string_b64 failed ! ***\n");
    else {
    printf("Plain text %s\n=\nBase64 %s\n", CSTR(tmp), CSTR(string));
    tmp = string_free(tmp);
    tmp = string_deb64(string);
    if (tmp == NULL)
    printf("! *** string_deb64 failed ! ***\n");
    else
    printf("Base64 %s =\nPlain text %s\n", CSTR(string), CSTR(tmp));
    }
    
    string = string_free(string); tmp = string_free(tmp);

    return 0;
}
