#include "../util/m_util_vfscanf.h"

/* gcc -g -Wall -std=c99 -posix -pedantic -O2 ../util/m_util_vfscanf.c ../util/m_util_float.c vfscanf.c -o vfscanf */

/* winegcc -mno-cygwin -I../ports/libpthread-win32/include -g -Wall -std=c99 -posix -pedantic -O2 ../util/m_util_vfscanf.c ../util/m_util_float.c vfscanf.c -o vfscanf */

#define u_quad_t unsigned long long

int main(void)
{
    const char string[] =
    "HTTP/1.1 200 OK\r\n"
    "Date: Wed, 28 Dec 2005 19:17:10 GMT\r\n"
    "Server: Apache/1.3.34 (Unix) PHP/5.0.5\r\n"
    "Content-Location: index.html.en\r\n"
    "Vary: negotiate,accept-language,accept-charset\r\n"
    "TCN: choice\r\n"
    "Last-Modified: Mon, 24 Oct 2005 19:08:21 GMT\r\n"
    "ETag: \"1131f-a71-435d3125;43ab5b00\"\r\n"
    "Accept-Ranges: bytes\r\n"
    "Content-Length: 2673\r\n"
    "Content-Type: text/html\r\n"
    "Content-Language: en\r\n"
    "\r\n";
    const size_t len = strlen(string);
    char buffer[256];
    int i = 0, r = 0;
    char int_buffer[sizeof(i)];
    /* 0xCAFEBADCODE in opposite endianness */
    u_quad_t ll = (u_quad_t) 0xDEC0ADEBAF0C0000ULL;
    char long_buffer[sizeof(ll)];
    double f = 1337.1234;
    char float_buffer[sizeof(f)];

    memset(buffer, 0, sizeof(buffer));

    /* test the Mammouth Extension '$' for variable field width */
    while ( (i = m_snscanf(string + r, len - r,"%$[^\r\n]%*c%*c", 255, buffer)) > 0) {
        r += i;
        printf("returns %i, buffer=%s\n", i, buffer);
        memset(buffer, 0, sizeof(buffer));
    }

    /* system integer test */
    i = 0xCAFE;
    printf("system integer : 0x%X\n", i);
    memcpy(int_buffer, (char *) & i, sizeof(i));
    i = 0;
    r = m_snscanf(int_buffer, sizeof(int_buffer), "%bi", & i);
    printf("returns %i, i = 0x%X\n", r, i);

    printf("long long, opposite endianness : 0x%llX\n", ll);
    memcpy(long_buffer, (char *) & ll, sizeof(ll));
    if ( (ll = 1) && (*(char *) & ll) == 1) {
        /* run on a little endian platform */
        r = m_snscanf(long_buffer, sizeof(long_buffer), "%bBlli", & ll);
    } else {
        /* big endian platform */
        r = m_snscanf(long_buffer, sizeof(long_buffer), "%bblli", & ll);
    }
    printf("returns %i, i = 0x%llX\n", r, ll);

    printf("double, big endian : %f\n", f);
    float_write_double(float_buffer, f, FLOAT_BIG); f = 0;
    r = m_snscanf(float_buffer, sizeof(long_buffer), "%bBlf", & f);
    printf("returns %i, i = %f\n", r, f);

    return 0;
}
