#ifndef M_QUEUE_H
 
#define M_QUEUE_H

#include "m_util_def.h"

/** @defgroup float util::float */

/** Read or write a big endian floating point value */
#define FLOAT_BIG    0x08
/** Read or write a little endian floating point value */
#define FLOAT_LITTLE 0x10

/* -------------------------------------------------------------------------- */

public inline float float_read_single(const char *buffer, int flags);

/**
 * @ingroup float
 * @fn float float_read_single(const char *buffer, int flags)
 * @param buffer the buffer to read the float from.
 * @param flags apply a specific treatment to the data read.
 * @return NAN if an error occurs, the float read otherwise.
 *
 * This function reads a binary single precision float from a buffer, using
 * the IEEE754 standard. The flags FLOAT_BIG or FLOAT_LITTLE indicate the
 * endianness of the binary data to the function.
 *
 * The function returns a float in the system endianness.
 *
 */

/* -------------------------------------------------------------------------- */

public inline double float_read_double(const char *buffer, int flags);

/**
 * @ingroup float
 * @fn double float_read_double(const char *buffer, int flags)
 * @param buffer the buffer to read the float from.
 * @param flags apply a specific treatment to the data read.
 * @return NAN if an error occurs, the double read otherwise.
 *
 * This function reads a binary double precision float from a buffer, using
 * the IEEE754 standard. The flags FLOAT_BIG or FLOAT_LITTLE indicate the
 * endianness of the binary data to the function.
 *
 * The function returns a double in the system endianness.
 *
 */

/* -------------------------------------------------------------------------- */

public inline int float_write_single(char *buffer, float f, int flags);

/**
 * @ingroup float
 * @fn int float_write_single(const char *buffer, float f, int flags)
 * @param buffer the buffer write the float to.
 * @param f the float to write.
 * @param flags apply a specific treatment to the data written.
 * @return -1 if an error occurs, 0 otherwise.
 *
 * This function writes a binary single precision float to the given buffer,
 * using the IEEE754 standard. The flags FLOAT_BIG or FLOAT_LITTLE indicate
 * the endianness with which the data should be written.
 *
 */

/* -------------------------------------------------------------------------- */

public inline int float_write_double(char *buffer, double f, int flags);

/**
 * @ingroup float
 * @fn int float_write_double(const char *buffer, double f, int flags)
 * @param buffer the buffer write the double to.
 * @param f the double to write.
 * @param flags apply a specific treatment to the data written.
 * @return -1 if an error occurs, 0 otherwise.
 *
 * This function writes a binary double precision float to the given buffer,
 * using the IEEE754 standard. The flags FLOAT_BIG or FLOAT_LITTLE indicate
 * the endianness with which the data should be written.
 *
 */

/* -------------------------------------------------------------------------- */

#endif
